(define-module (ui controlpp edit ajax)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :use-module (alterator effect)
  :export (init))

(define (mode)
  (if (not (eq? (form-value "id") #f)) "edit" "add"))

(define (string-replace-char str c1 s2)
  (string-join (string-split str c1) s2))

(define (page2offset page)
  (* 25 (- page 1)))

(define (current-page)
  (let ((page (form-value "pager")))
    (string->number (if (string=? page "") "1" page))))

(define (on-create-click)
  (begin
    (catch/message (lambda()
      (form-update-value-list
        (woo "create" "/controlpp"
          'summary (form-value "summary")
          'autogen (form-value "autogen")
          'ltype (form-value "ltype")
          'language (form-value "language")))))
    (ui-read-list 0 "")
    (form-update-value "mode" (mode))
    (update-effect)))

(define (on-save-click)
  (begin
    (catch/message (lambda()
      (form-update-value-list
        (woo "write" "/controlpp"
          'id (form-value "id")
          'summary (form-value "summary")
          'language (form-value "language")))))
    (form-replace "/controlpp/lists")))

(define (on-cancel-click)
  (form-replace "/controlpp/lists"))

(define (ui-read-list offset filter)
  (begin
    (form-update-value-list
      (woo-read-first "/controlpp"
        'id (form-value "id")
        'filter filter
        'language (form-value "language")))
    (form-update-enum "paths"
      (woo-list "/controlpp"
        'id (form-value "id")
        'offset offset
        'filter filter
        'language (form-value "language")))))

(define (ui-reload-list)
  (ui-read-list 
    (page2offset (current-page))
    (form-value "pathfilter")))

(define (on-goto-click)
  (begin
    (let
      ((offset (page2offset (current-page)))
      (filter (form-value "pathfilter")))

      (ui-read-list offset filter))
    (update-effect)))
  
(define (on-filter-click)
  (begin
    (ui-read-list 0 (form-value "pathfilter"))
    (form-update-value "pager" 1)
    (update-effect)))

(define (on-reset-filter-click)
  (begin
    (form-update-value "pathfilter" "")
    (form-update-value "pager" 1)
    (ui-read-list 0 "")
    (update-effect)))

(define (on-add-click)
  (begin
    (catch/message (lambda()
      (form-update-value-list
        (woo "append_path" "/controlpp"
          'id (form-value "id")
          'path (form-value "path")
          'language (form-value "language")))
      (form-update-value "path" "")
      (ui-reload-list)
      (update-effect)))))

(define (on-remove-click)
  (begin
    (catch/message (lambda() 
      (form-update-value-list
        (woo "remove_path" "/controlpp"
          'id (form-value "id")
          'path (form-value "paths")
          'language (form-value "language")))
      (ui-reload-list)
      (update-effect)))))

(define (on-enable-click)
  (begin
    (catch/message (lambda() 
      (form-update-value-list
        (woo "enable_path" "/controlpp"
          'id (form-value "id")
          'path (form-value "paths")
          'language (form-value "language")))
      (ui-reload-list)
      (update-effect)))))

(define (on-disable-click)
  (begin
    (catch/message (lambda() 
      (form-update-value-list
        (woo "disable_path" "/controlpp"
          'id (form-value "id")
          'path (form-value "paths")
          'language (form-value "language")))
      (ui-reload-list)
      (update-effect)))))

(define (goto-page page)
  (let
    ((offset (page2offset page))
    (filter (form-value "pathfilter")))
      (begin
          (ui-read-list offset filter)
          (form-update-value "pager" page)
          (update-effect))))

(define (on-nextpage-click)
  (let ((total-pages (string->number (form-value "pages"))))
    (if (< (current-page) total-pages)
      (goto-page (+ (current-page) 1)))))

(define (on-prevpage-click)
  (if (> (current-page) 1)
    (goto-page (- (current-page) 1))))

(define (on-firstpage-click)
  (goto-page 1))

(define (on-lastpage-click)
  (let ((total-pages (string->number (form-value "pages"))))
    (goto-page total-pages)))

(define (is-active)
  (woo-get-option (woo-read-first "/controlpp/status") 'is_active))

(define (init-form)
  (form-update-value "id" (form-value "_id"))
  (form-bind "create" "click" on-create-click)
  (form-bind "save" "click" on-save-click)
  (form-bind "cancel" "click" on-cancel-click)
  (form-bind "goto" "click" on-goto-click)
  (form-bind "search" "click" on-filter-click)
  (form-bind "add" "click" on-add-click)
  (form-bind "remove" "click" on-remove-click)
  (form-bind "enable" "click" on-enable-click)
  (form-bind "disable" "click" on-disable-click)
  (form-bind "nextpage" "click" on-nextpage-click)
  (form-bind "prevpage" "click" on-prevpage-click)
  (form-bind "firstpage" "click" on-firstpage-click)
  (form-bind "lastpage" "click" on-lastpage-click)
  (form-bind "reset-filter" "click" on-reset-filter-click)
  (form-update-value "mode" (mode))
  (form-update-value "pager" 1)
  (effect-hide "editor" "mode" "add")
  (effect-hide "additor" "mode" "edit")
  (effect-hide "whitelist-options" "ltype" "blacklist")
  (effect-hide "create" "mode" "edit")
  (effect-hide "save" "mode" "add")
  (init-effect)
  (if (string= (mode) "edit")
    (begin
      (ui-read-list 0 "")
      (update-effect))))

(define (init)
  (if (is-active)
    (init-form)
    (form-replace "/controlpp/setup")))
