(define-module (ui controlpp lists ajax)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :use-module (ice-9 regex)
  :export (init))

(define (selected-list)
  (form-value "lists"))

(define (ui-list)
  (form-update-enum "lists"
    (woo-list "/controlpp/lists" 'language (form-value "language"))))

; (define (log msg)
;   (format (current-error-port) (string-append "debug-message: " msg "\n")))

(define (on-apply-click)
  (let ((id (selected-list)))
    (if (not id)
      (form-error (_ "Please, select the list"))
      (let* ((lang (form-value "language"))
            (listinfo (woo-read-first "/controlpp" 'id id 'filter "" 'language lang))
            (danger (woo-get-option listinfo 'danger))
            (warnlist (woo-get-option listinfo 'warnlist))
            (warnmsg (string-append
              (_ "This list seems to be dangerous. ")
              (_ "The following executables will be disabled: ")
              (regexp-substitute/global #f "\n" warnlist 'pre ", " 'post)
              (_ ". It can be impossible to change any permissions later ")
              (_ "(to reset the permission mode back, for example) ")
              (_ "or it may brake the alterator or the alterator-control++ module. ")
              (_ "Do you really want to apply this list?"))))
        (if danger
          (if (form-confirm warnmsg)
            (apply-list-and-refresh list))
          (apply-list-and-refresh id))))))

(define (apply-list-and-refresh id)
  (catch/message (lambda ()
    (woo "apply" "/controlpp" 'id id
      'language (form-value "language"))))
  (ui-list))

(define (on-reset-click)
  (catch/message (lambda ()
    (woo "reset" "/controlpp" 'id (selected-list)
      'language (form-value "language"))))
  (ui-list))

(define (on-add-click)
  (form-replace "/controlpp/edit"))

(define (on-delete-click)
  (catch/message (lambda ()
    (woo "remove" "/controlpp" 'id (selected-list)
      'language (form-value "language"))))
  (ui-list))

(define (on-edit-click)
  (let ((list (selected-list)))
    (if (eq? list #f)
      (form-error (_ "Please, select the list"))
      (if (is-active-list list)
        (form-error (_ "Can't edit the activated list"))
        (form-replace "/controlpp/edit" '_id list)))))

(define (is-dangerous-list id)
  (let ((lang (form-value "language")))
    (woo-get-option (woo-read-first "/controlpp"
      'id id 'filter "" 'language lang) 'danger)))

(define (is-active-list id)
  (woo-get-option (woo-read-first "/controlpp"
    'id id 'filter "" 'language (form-value "language")) 'active))

(define (is-active)
  (woo-get-option (woo-read-first "/controlpp/status") 'is_active))

(define (init-form)
  (form-bind "add" "click" on-add-click)
  (form-bind "delete" "click" on-delete-click)
  (form-bind "edit" "click" on-edit-click)
  (form-bind "apply" "click" on-apply-click)
  (form-bind "reset" "click" on-reset-click)
  (ui-list))

(define (init)
  (if (is-active)
    (init-form)
    (form-replace "/controlpp/setup")))
